import java.util.Arrays;
import java.util.List;

import de.raida.cad.dex.cadimport.adapter.EntityContainer;
import de.raida.cad.dex.cadimport.obj.OBJImporterImplementation;
import de.raida.cad.dex.interfaces.CADImportInterface;

/**
 * Example class for loading and a CAD file and viewing it's attributes.
 * @author  <a href="mailto:j.raida@gmx.net">Johannes Raida</a>
 * @version 1.0
 */
public class LoadOBJ {
	/**
	 * Constructor, creating the scene.
	 * @param fileName File name
	 */
	public LoadOBJ(String fileName){
		try {
			// Load an OBJ file
			CADImportInterface cadImportInterface = new OBJImporterImplementation();
			cadImportInterface.importFile(fileName, null);

			// Print some meta data
			System.out.println("\nNumber of layers: " + cadImportInterface.getLayerMetaData().size());

			System.out.println("\nExtreme values:");
			double[][] extremeValues = cadImportInterface.getExtremeValues();
			System.out.println("   Minimum: " + extremeValues[0][0] + ", " + extremeValues[0][1] + ", " + extremeValues[0][2]);
			System.out.println("   Maximum: " + extremeValues[1][0] + ", " + extremeValues[1][1] + ", " + extremeValues[1][2]);

			System.out.println("\nViewport:");
			double[][] viewPort = cadImportInterface.getViewPort();
			if(viewPort != null){
				System.out.println("   Minimum: " + viewPort[0][0] + ", " + viewPort[0][1] + ", " + viewPort[0][2]);
				System.out.println("   Maximum: " + viewPort[1][0] + ", " + viewPort[1][1] + ", " + viewPort[1][2]);
			}

			System.out.println("\nAdditional information:");
			List<String[]> modelInformation = cadImportInterface.getAdditionalModelInformation();
			for(int i = 0; i < modelInformation.size(); i++){
				String[] information = modelInformation.get(i);
				if(!information[0].equals("") && !information[1].equals("")){
					System.out.println("   " + information[0] + ": " + information[1]);
				} else if(!information[0].equals("")){
					System.out.println("   " + information[0]);
				} else if(!information[1].equals("")){
					System.out.println("   " + information[1]);
				} else {
					System.out.println();
				}
			}

			System.out.println("\nLoad information:");
			List<String[]> loadInformation = cadImportInterface.getImportInformation();
			for(int i = 0; i < loadInformation.size(); i++){
				String[] information = loadInformation.get(i);
				System.out.println("   " + information[0] + ": " + information[1]);
			}

			System.out.println("\nUnsupported entities:");
			List<String> unsupportedEntities = cadImportInterface.getUnsupportedEntities();
			for(int i = 0; i < unsupportedEntities.size(); i++){
				String unsupportedEntity = unsupportedEntities.get(i);
				System.out.println("   " + unsupportedEntity);
			}

			// Print the information about the CAD entities
			System.out.println("\nCAD entities:");
			List<EntityContainer> entityContainers = cadImportInterface.getEntities();
			for(int i = 0; ((i < entityContainers.size()) && (i != 1)); i++){
				EntityContainer entityContainer = entityContainers.get(i);

				System.out.println("  Node name:      " + entityContainer.getNodeName());
				switch(entityContainer.getType()){
					case EntityContainer.POINTS:
						System.out.println("  Entity type:    POINTS");
						break;
					case EntityContainer.POLYLINE:
						System.out.println("  Entity type:    POLYLINE");
						break;
					case EntityContainer.TRIANGLES:
						System.out.println("  Entity type:    TRIANGLES");
						break;
					default:
						System.out.println("  Entity type:    Unknown");
				}
				System.out.println("  Vertex colors:  " + Arrays.toString(entityContainer.getVertexColors()));
				System.out.println("  Vertex indices: " + Arrays.toString(entityContainer.getVertexIndices()));
				System.out.println("  Vertex normals: " + Arrays.toString(entityContainer.getVertexNormals()));
				System.out.println("  Vertices:       " + Arrays.toString(entityContainer.getVertices()));
			}

		} catch(Exception exception){
			exception.printStackTrace();
		}
	}

	/**
	 * Entry point of the class
	 * @param arguments Arguments, given by the command line
	 */
	public static void main(String[] arguments){
		if(arguments.length != 1){
			System.out.println("Wrong number of parameter!");
			System.out.println("Usage: java LoadOBJ <OBJ file>");
			return;
		}

		new LoadOBJ(arguments[0]);
	}
}